/***********************************************************************
(c) 1994-1999 Imperial College, F.G. McCabe  and Fujitsu Labs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Contact: fgm@fla.fujitsu.com
***********************************************************************/

/*
 * I/O formatting functions for the ICM library
 */

#include "config.h"		/* Invoke configuration header */
#include "icmP.h"
#include "icmIoP.h"
#include <stdarg.h>
#ifdef WIN32
#include <winsock.h>
#else
#include <sys/time.h>		/* This is where FD_SETSIZE is defined!!!! */
#endif
#include <float.h>		/* For fp conversion */
#include <math.h>
#include <ctype.h>
#include <string.h>

static char *Int2StrByBase(char *str,Ullong i,int base);

icmStatus outInt(filePo f,llong i)
{
  char buff[64];
  int pos = 0;

  if(i<0){
    buff[pos++]='-';
    i = -i;
  }

  Int2StrByBase(&buff[pos],(Ullong)i,10);

  return icmOutStr(f,buff);
}

#ifdef WIN32
static char hxDgit (Ulong h)
#else
static inline char hxDgit (Ulong h)
#endif
{
  if(h<10)
    return h|'0';
  else
    return h+'A'-10;
}

static char *Int2StrByBase(char *str,Ullong i,int base)
{
  if(i<base)
    *str++=hxDgit(i);
  else{
    str=Int2StrByBase(str,i/base,base);
    *str++=hxDgit(i%base);
  }
  *str='\0';
  return str;
}

icmStatus int2Str(llong i,char *buff,long len)
{
  if(i<0){
    *buff='-';
    if(Int2StrByBase(buff+1,-i,10)-buff>=0)
      return icmOk;
    else
      return icmError;
  }
  else if(Int2StrByBase(buff,i,10)-buff>=0)
    return icmOk;
  else
    return icmError;
}

icmStatus outInteger(filePo f,llong i,int base,int width,int precision,
		     char pad,icmTruth left,char *prefix,
		     icmTruth sign)
{
  char iBuff[64];
  int len;

  if(i>=0 && sign)
    prefix="+";
  else if(i<0){
    prefix="-";
    i = -i;
  }

  if(!left)
    pad=' ';			/* We dont put trailing zeroes */

  len = Int2StrByBase(iBuff,i,base)-iBuff;

  return outString(f,iBuff,len,width,precision,pad,left,prefix);
}


static icmStatus outOctal(filePo f,long i,int width,int precision,char pad,
			  icmTruth left,char *prefix,icmTruth sign,icmTruth alt)
{
  char iBuff[64];
  int len = Int2StrByBase(iBuff,i,8)-iBuff;

  if(i>=0 && sign)
    prefix="+";

  if(!left)
    pad=' ';			/* We dont put trailing zeroes */

  return outString(f,iBuff,len,width,precision,pad,left,prefix);
}

static icmStatus outHex(filePo f,long i,int width,int precision,char pad,
			icmTruth left,char *prefix,icmTruth sign,icmTruth alt)
{
  char iBuff[64];
  int len = Int2StrByBase(iBuff,i,16)-iBuff;

  if(alt)
    prefix="0x";

  if(!left)
    pad=' ';			/* We dont put trailing zeroes */

  return outString(f,iBuff,len,width,precision,pad,left,prefix);
}

/* Convert a floating point to decimal format */

static const double bit_values[] = {
  1.0E1, 1.0E2, 1.0E4, 1.0E8, 1.0E16, 1.0E32, 1.0E64, 1.0E128, 1.0E256
};

#define max_bits 511
  
static int number2Str(double x,int precision,char *dec,int *exp)
{
  int exp2;
  long exp10;
  int len=0;
  char *digits = dec;
	
/*
 *	We first deal with the special cases of zeroes, infinities and NaNs
 */

  *exp = 0;
	
  if(x==0.0)
    return strlen(strcpy(digits,"0"));
  else if(x>DBL_MAX)
    return strlen(strcpy(digits,"Infinity"));
  else{
    frexp(x, &exp2);		/* Get the scale of the number */
	
    exp10 = exp2*log10(FLT_RADIX); /* Convert exponent to base10 exponent  */
	
    {				/* Fast way of scaling number */
      const double *p = bit_values;
      int n = exp10;
		
      if(n<0){			/* We have a number smaller than 10 */
	for (n = -n; n; p++, n >>= 1)
	  if (n & 1)
	    x *= *p;
      }
      else if(n>0){
	double f = 1.0;
			
	for (; n; p++, n >>= 1)
	  if(n & 1)
	    f *= *p;
	x /= f;
      }
    }
	
    while(x>=1.0){
      x *= 0.1;
      ++exp10;
    }
	
    while(x<0.1){
      x *= 10.0;
      --exp10;
    }

    while(precision-->=0){	/* Go to one more than required precision */
      double front;

      x = modf(x*10,&front);	/* extract left-most digit */

      *digits++=hxDgit((int)front);
      len++;
    }

    *digits--='\0';		/* Terminate digit string */

    if(*digits>'5'){		/* start rounding */
      digits--;
      while(digits>dec){
	if(*digits=='9')
	  *digits--='0';
	else{
	  (*digits)++;		/* increment last digit */
	  break;		/* and stop the loop */
	}
      }
    }
    else
      *digits='0';
    *exp = exp10;
    return len-1;		/* we generated an extra digit */
  }
}

icmStatus outDouble(filePo out,double x,char mode,int width,int precision,
		    char pad,icmTruth left,char *prefix,icmTruth sign)
{
  char dec[DBL_DIG*2];		/* buffer for the decimal mantissae */
  char *d = dec;
  char buff[1024];		/* buffer to represent the number string */
  char *p = buff;
  char *eP = &buff[NumberOf(buff)-1]; /* end marker */

  int exp,len,sig;

  if(x<0){
    prefix="-";
    x = -x;
  }
  else if(sign)
    prefix="+";			/* Is the number signed? */
  
  len = sig = number2Str(x,DBL_DIG,dec,&exp);

  while(sig>1 && dec[sig-1]=='0')
    sig--;			/* chop off trailing zeroes */

  if(strcmp(dec,"Infinity")==0)
    return outString(out,dec,strlen(dec),width,precision,pad,left,prefix);
  else{
    if(tolower(mode)=='e'||(tolower(mode)=='g' && (exp<-3||exp>sig+1))){
      *p++=*d++;			/* use scientific format */
      len--; sig--;
      if(precision>0){
	*p++='.';
	while(precision-->0)
	  if(len-->0)
	    *p++=*d++;
	  else
	    *p++=*d;		/* trailing zero */
      }
      else if(precision==0 && len>=0){
	*p++='.';
	while(sig-->0)
	  *p++=*d++;
      }

      *p++='E';			/* Show exponent sign */
      if(--exp<0){
	*p++='-';
	exp = -exp;
      }
      Int2StrByBase(p,exp,10);/* Show exponent value -- adjusted for leading digit*/
    }
    else if(exp<=0){		/* Use fixed point format */
      int prec = precision;

      *p++='0';
      *p++='.';

      if(precision==0)
	while(p<eP && exp++<0)
	  *p++='0';
      else
	while(precision-->0 && p<eP && exp++<0)
	  *p++='0';

      if(prec!=0){
	while(p<eP && precision-->0)
	  if(len-->0)
	    *p++=*d++;
	  else
	    *p++='0';
      }
      else{			/* display all available digits */
	while(p<eP && sig-->0)
	  *p++=*d++;
      }
      *p='\0';
    }
    else{
      while(p<eP && exp-->0)
	if(len-->0){
	  *p++=*d++;
	  sig--;
	}
	else
	  *p++=*d;

      if(p<eP && precision>0){
	*p++='.';
	while(p<eP && precision-->0)	/* copy out the fractional part */
	  if(len-->0)
	    *p++=*d++;
	  else
	    *p++='0';
      }
      else if(p<eP && precision==0){
	if(sig>0){
	  *p++='.';
	  while(p<eP && sig-->0)	/* copy out the fractional part */
	    *p++=*d++;
	}
      }
      *p='\0';
    }

    return outString(out,buff,strlen(buff),width,strlen(buff),pad,left,prefix);
  }
}

icmStatus outFloat(filePo out,double x)
{
  return outDouble(out,x,'g',0,0,' ',icmYes,"",icmNo);
}


icmStatus icmOutStr(filePo f,char *str)
{
  while(*str!='\0')
    outCh(f,*str++);
  return icmOk;
}

icmStatus outString(filePo f,char *str,int len,int width,int precision,
		    char pad,icmTruth leftPad,char *prefix)
{
  int gaps;

  outBlock(f,prefix,strlen(prefix)); /* write out the prefix string */

  if(precision>0 && precision<len)
    len = precision;		/* we only show part of the string */

  if(width>0){			/* fixed width */
    if(len>width)
      len=width;		/* never print more than available width */

    if(!leftPad){		/* right justified */
      gaps = width-len;

      outBlock(f,str,len);

      while(gaps-->0)
	outCh(f,pad);
    }
    else{
      gaps = width-len;

      while(gaps-->0)
	outCh(f,pad);
      outBlock(f,str,len);
    }
  }
  else
    outBlock(f,str,len);	/* variable width */
  return icmOk;
}

/**********************************************************************/
/*               Display a formatted message                          */
/**********************************************************************/

static fileMsgProc procs[256];		/* What to invoke... */

void _installMsgProc(char key,fileMsgProc proc)
{
  initICMIo();
  procs[(unsigned int)key] = proc;
}

void _initMsgProcs_(void)
{
  int i;
  for(i=0;i<255;i++)
    procs[i]=NULL;
}


/* We have our own version of fprintf too */

/* This one is used in april-log_msg */  
icmStatus __voutMsg(filePo f,char *fmt,va_list args)
{
  while(*fmt!='\0'){
    switch(*fmt){
    case '%':{
      char c;
      long width=0;		/* Maximum width of field */
      long precision=0;		/* Minimum width or precision of field */
      char pad=' ';
      char *prefix="";
      icmTruth sign=icmNo;
      icmTruth alternate=icmNo;
      icmTruth leftPad = icmYes;
      icmTruth overridePrecision = icmNo;
      icmTruth longValue=icmNo;

      fmt++;

      while(strchr("0 -#+",*fmt)!=NULL){
	switch(*fmt++){
	case '0':
	  pad = '0';
	  continue;
	case ' ':
	  prefix=" ";
	  continue;
	case '+':
	  sign=icmYes;
	  continue;
	case 'l':
	  longValue=icmYes;
	  continue;
	case '#':
	  alternate=icmYes;
	  continue;
	case '-':
	  leftPad=icmNo;
	  continue;
	}
      }

      while(isdigit((unsigned char)c=*fmt++))	/* extract the width field */
	width = width*10+(c&0xf);

      if(c=='.'){		/* We have a precision ... */
	overridePrecision=icmYes;
	while(isdigit((unsigned char)c=*fmt++))
	  precision = precision*10+(c&0xf);
      }

      if(procs[(unsigned int)c]!=NULL){
	void *data = (void *)va_arg(args, void*); /* pick up a special value */
	procs[(unsigned int)c](f,data,width,precision,alternate);
      }
      else
	switch(c){
	case 'c':{		/* Display an integer value as a char */
	  llong i = (llong)(longValue?va_arg(args,llong):va_arg(args,long)); 
	  
	  outCh(f,i);
	  break;
	}
	case 'd':{		/* Display an integer value */
	  llong i = (llong)(longValue?va_arg(args,llong):va_arg(args,long)); 
	  
	  outInteger(f,i,10,width,precision,pad,leftPad,prefix,sign);
	  break;
	}
	case 'u':{		/* Display a number as unsigned */
	  Ullong i = (Ullong)(longValue?va_arg(args,Ullong):va_arg(args,unsigned long)); 
	  char iBuff[64];
	  int len;

	  if(!leftPad)
	    pad=' ';		/* We dont put trailing zeroes */

	  len = Int2StrByBase(iBuff,i,10)-iBuff;

	  outString(f,iBuff,len,width,precision,pad,leftPad,prefix);
	  break;
	}
	case 'o':{		/* Display an octal value */
	  llong i = (llong)(longValue?va_arg(args,llong):va_arg(args,long)); 

	  outOctal(f,i,width,precision,pad,leftPad,prefix,sign,alternate);
	  break;
	}
	case 'X':
	case 'x':{		/* Display a hex value */
	  llong i = (llong)(longValue?va_arg(args,llong):va_arg(args,long)); 

	  outHex(f,i,width,precision,pad,leftPad,prefix,sign,alternate);
	  break;
	}
	case 'g':
	case 'G':
	case 'e':
	case 'E':	
	case 'F':
	case 'f':{		/* Display floating point number */
	  double num = (double)va_arg(args,double);

	  if(!overridePrecision) /* default precision for floats */
	    precision=6;
	  outDouble(f,num,c,width,precision,pad,leftPad,prefix,sign);
	  break;
	}
	case 'P':{			/* Display a Pascal string */
	  char *str = (char*)va_arg(args,char *);
	  
	  if(str!=NULL){
	    int len = *str++;

	    outString(f,str,len,width,precision,' ',leftPad,prefix);
	  }
	  else
	    icmOutStr(f,"(NULL)");
	  break;
	}
	case 's':{		/* Display a string */
	  char *str = (char*)va_arg(args,char *);
	  
	  if(str!=NULL)
	    outString(f,str,strlen(str),width,precision,' ',leftPad,prefix);
	  else
	    icmOutStr(f,"(NULL)");
	  break;
	}

	case 'S':{		/* Display a data block */
	  long len = (long)va_arg(args,long);
	  char *str = (char*)va_arg(args,char *);
	  
	  if(str!=NULL){
	    int i;

	    for(i=0;i<len;i++)
	      if(isprint((unsigned char)str[i]))
		outCh(f,str[i]);
	      else
		outMsg(f,"\\%x\\",str[i]&0xff);
	  }
	  else
	    icmOutStr(f,"(NULL)");
	  break;
	}

	default:
	  outCh(f,c);
	}
      break;
    }
    
    default:
      outCh(f,*fmt++);
    }
  }
  return icmOk;
}

icmStatus icmOutMsg(filePo f,char *fmt,...)
{
  if(f!=NULL){
    va_list args;		/* access the generic arguments */
    va_start(args,fmt);		/* start the variable argument sequence */

    __voutMsg(f,fmt,args);

    va_end(args);
    return icmOk;
  }
  else
    return icmError;
}

/* Legacy version */
icmStatus outMsg(filePo f,char *fmt,...)
{
  if(f!=NULL){
    va_list args;		/* access the generic arguments */
    va_start(args,fmt);		/* start the variable argument sequence */

    __voutMsg(f,fmt,args);

    va_end(args);
    return icmOk;
  }
  else
    return icmError;
}


icmStatus outStr(filePo f,char *str)
{
  return icmOutStr(f,str);
}

