#ifndef _ICMP_H_
#define _ICMP_H_

#ifdef WIN32
#include <time.h>
#else
#include <sys/time.h>
#endif

/* Private header file for the ICM */
#ifndef _REENTRANT
#define _REENTRANT		/* Signal for re-entrant version of libraries */
#endif

#include "icm.h"		/* Make sure we have the public stuff */
#include "icmIo.h"
#include "icmFile.h"

#ifdef _WIN32			/* Windows'95 specific stuff */
#include <windows.h>
#else
#ifdef __MWERKS__		/* Code Warrior specific stuff */
#include <winsock.h>		/* Winsock interface */
#else
#include <netdb.h>
#endif
#endif

#ifndef MAXSTRING
#define MAXSTRING 1024
#endif

#ifndef MAXPACKET
#define MAXPACKET 1534
#endif

#ifndef ICMCOMMSPORT
#define ICMCOMMSPORT 4549
#endif

#ifndef NumberOf
#define NumberOf(x) (sizeof(x)/sizeof(x[0]))
#endif

typedef unsigned char Uchar;

typedef unsigned long Ulong;
typedef unsigned long long Ullong;

typedef struct _icmMessage_ {
  long used;
  long size;			/* Size of the message buffer */
  Uchar buffer[MAXSTRING];	/* This is to avoid unnecessary mallocs & frees */
  Uchar *data;
} icmMsgRec;

typedef struct _icmMsgList_ *icmMsgListPo;

typedef struct _msg_pkts_ *packetPo;

typedef struct _msg_pkts_ {
  long pNo;			/* What is the packet number? */
  Uchar *data;			/* What is the data content of this packet */
  long dlen;			/* Length of this data block */
  packetPo next;
} MsgPacket;

typedef struct _icmMsgList_ {
  icmHandle sndr;		/* Sender of this message */
  long msgNo;			/* Index key for this message */
  long count;			/* Number of packets in this message */
  long outstanding;		/* Number of packets left to collect */
  icmMsg msg;			/* Message being contructed */
  packetPo packets;		/* Packets collected so far */
  long seqNo;			/* What is the sequence number of this message */
  time_t lease;			/* when does this message time out? */
  icmMsgListPo next;		/* Next message being constructed */
} icmMsgListRec;

typedef struct _icm_msg_option_ {
  icmOption next;		/* next option in the sequence */
  icmOpt opt;			/* tag identifying the option */
  union {
    time_t when;		/* when does the lease run out? */
    icmHandle reply;		/* who to reply to */
    llong code;			/* code used in acknowledging message receipt */
    icmHandle *trail;		/* Audit trail -- array of icm handles */
  } o;
} icmOptRecord;

typedef struct _icmConnection_ {
  char host[MAXSTRING];		/* Host name of the connection */
  int port;			/* Port used to make the connection */
  filePo conn;			/* File associated with the connection */
  int socket;			/* Socket number associated with the connection */
  icmMsgListPo pending;		/* Messages that havent been completely read in */
  icmHandle server;		/* Name of this communications server */
  icmIdleProc idler;		/* PRocedure called during the message loop */
  void *idle_client;
  long seqNo;			/* Sequence number of the next message */
  icmHandle dummyH;		/* Used for some API functions */
  pthread_mutex_t mutex;	/* Avoid threads accessing connection simultaneously */
  icmTruth loopGoing;		/* true while in the event loop */
} icmConnRec;

/* Host machine identification management */
void initICMHosts(void);
void icmMarkHostUnavail(char *name);

icmStatus _CallCallBack_(icmConn conn,icmHandle t,icmHandle s,icmOption o,icmMsg m);
void icmCheckHandle(icmHandle h);

/* Standard file to write messages to */
extern filePo icmLogFile;

icmConn icmMakeConn(filePo rem,int port,char *host);
icmMsg icmPacketizeMsg(icmMsg msg,icmHandle recip,icmHandle owner);
long icmMsgLength(icmMsg msg);
void icmResetMsg(icmMsg m);
void icmClearMsg(icmMsg m);
icmStatus icmCheckMsg(icmMsg msg);
Uchar *icmMsgData(icmMsg msg);
void icmSetMsg(icmMsg m,Uchar *data,long size);

void icmDumpIO(filePo f);
icmStatus icmPutBlock(Uchar *bytes,long len,icmMsg msg);

icmStatus icmMakeComms(char *host,int port,filePo file,icmConn *conn);

void icmDumpAddresses(void);

#ifndef ICM_COMMSERVER
#define ICM_COMMSERVER "icmCommServer"
#endif

#ifdef ALLTRACE

#ifndef COMMTRACE
#define COMMTRACE
#endif

#ifndef MUTEXTRACE
#define MUTEXTRACE
#endif

#ifndef WKRTRACE
#define WKRTRACE
#endif

#ifndef HANDLETRACE
#define HANDLETRACE
#endif

#ifndef WKRTRACE
#define WKRTRACE
#endif

#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

#ifdef MUTEXTRACE
extern icmTruth traceMutex;	/* True if we are tracing mutexes */
#endif

#ifdef HANDLETRACE
extern icmTruth traceHandle;
#endif

#ifdef WKRTRACE
extern icmTruth traceWkr;
#endif

#ifndef BOLD_ESC_ON
#define BOLD_ESC_ON "\033[5m"
#define BOLD_ESC_OFF "\033[0m"
#endif

#endif
