/***********************************************************************
(c) 1994-1999 Imperial College, F.G. McCabe  and Fujitsu Labs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Contact: fgm@fla.fujitsu.com
***********************************************************************/

/*
 * Glue and interface to pthreads mutexes
 *
 */

#include "config.h"		/* Invoke configuration header */
#include "icmP.h"		/* Private interface */
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#ifdef MUTEXTRACE
icmTruth traceMutex=icmNo;	/* True if we are tracing mutexes */
#endif


void icmCreateMutex(pthread_mutex_t *mutex)
{
  pthread_mutexattr_t attr;

  pthread_mutexattr_init(&attr);

#ifdef MUTEXTRACE_
  pthread_mutexattr_setkind_np(&attr,PTHREAD_MUTEX_ERRORCHECK_NP);
#endif

  if(pthread_mutex_init(mutex,&attr)!=0){
    icmLogMsg("%d: (0x%x) problem in creating mutex",pthread_self(),(Ulong)mutex);
    exit(1);
  }

  pthread_mutexattr_destroy(&attr);
}

void icmDestroyMutex(pthread_mutex_t *mutex)
{
  pthread_mutex_unlock(mutex);
  if(pthread_mutex_destroy(mutex)!=0){
    icmLogMsg("%d: (0x%x) problem in destroying mutex",pthread_self(),
	      (Ulong)mutex);
    exit(1);
  }
}

void icmLockMutex(pthread_mutex_t *mutex)
{
  if(pthread_mutex_lock(mutex)!=0){
    switch(errno){
    case EINVAL:
      icmLogMsg("%d: thread priority exceeds mutex priority ceiling",pthread_self());
      exit(1);
    case EBUSY:
      icmLogMsg(BOLD_ESC_ON "%d: mutex already locked, waiting..."BOLD_ESC_OFF,
		pthread_self());
      pthread_mutex_lock(mutex);
      return;
    case EDEADLK:
      icmLogMsg("%d: mutex already locked, by owner...",pthread_self());
      exit(1);
    default:
      icmLogMsg("%d: problem (%s) in locking mutex...",pthread_self(),strerror(errno));
      exit(1);

    }
  }
}

void icmUnlockMutex(pthread_mutex_t *mutex)
{
  if(pthread_mutex_unlock(mutex)!=0){
    icmLogMsg("%d: (0x%x) problem in unlocking mutex",pthread_self(),(Ulong)mutex);
    exit(1);
  }
}

void icmWaitCond(pthread_cond_t *cond,pthread_mutex_t *mutex)
{
  if(pthread_cond_wait(cond,mutex)!=0){
    icmLogMsg("Problem in waiting");
    exit(1);
  }
}

void icmSignalCond(pthread_cond_t *cond)
{
  if(pthread_cond_signal(cond)!=0){
    icmLogMsg("Problem in signalling condition");
    exit(1);
  }
}

void icmBroadcastCond(pthread_cond_t *cond)
{
  if(pthread_cond_broadcast(cond)!=0){
    icmLogMsg("Problem in broadcasting condition");
    exit(1);
  }
}

