/***********************************************************************
(c) 1994-1999 Imperial College, F.G. McCabe  and Fujitsu Labs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Contact: fgm@fla.fujitsu.com
***********************************************************************/

/*
 * Reader/Writer locks
 */
#include "config.h"		/* Invoke configuration header */
#include "icmP.h"		/* Private interface */

typedef struct _rw_lock_ {
  pthread_mutex_t mutex;
  pthread_cond_t rready;
  pthread_cond_t wready;
  long wCnt;
  long rCnt;
} LockRec;

icmPoolBase rwPool = NULL;

static pthread_once_t lockInit = PTHREAD_ONCE_INIT;

static void init_locks(void)
{
  rwPool = icmInitPool(sizeof(LockRec),64);
}

icmLockPo icmNewLock(void)
{
  icmLockPo lock;

  pthread_once(&lockInit,init_locks);

  lock = icmAllocPool(rwPool);

  lock->wCnt = lock->rCnt = 0;
  icmCreateMutex(&lock->mutex);
  pthread_cond_init(&lock->rready,NULL);
  pthread_cond_init(&lock->wready,NULL);

  return lock;
}

void icmReleaseLock(icmLockPo lock)
{
  icmLockMutex(&lock->mutex);

  while(lock->wCnt>0)
    pthread_cond_wait(&lock->rready,&lock->mutex);
  while(lock->rCnt>0)
    pthread_cond_wait(&lock->wready,&lock->mutex);

  icmUnlockMutex(&lock->mutex);

  pthread_mutex_destroy(&lock->mutex);
  pthread_cond_destroy(&lock->rready);
  pthread_cond_destroy(&lock->wready);
}

void icmReadLock(icmLockPo lock)
{
  icmLockMutex(&lock->mutex); /* acquire the mutex */

  while(lock->wCnt>0)
    pthread_cond_wait(&lock->rready,&lock->mutex);

  lock->rCnt++;
  icmUnlockMutex(&lock->mutex);
}

void icmReadUnLock(icmLockPo lock)
{
  icmLockMutex(&lock->mutex); /* acquire the mutex */

  lock->rCnt--;
  pthread_cond_signal(&lock->wready); /* signal a writer */

  icmUnlockMutex(&lock->mutex);
}

void icmWriteLock(icmLockPo lock)
{
  icmLockMutex(&lock->mutex); /* acquire the mutex */

  while(lock->rCnt>0)
    pthread_cond_wait(&lock->wready,&lock->mutex);

  lock->wCnt++;
  icmUnlockMutex(&lock->mutex);
}

void icmWriteUnLock(icmLockPo lock)
{
  icmLockMutex(&lock->mutex); /* acquire the mutex */

  if(--lock->wCnt==0)
    pthread_cond_broadcast(&lock->rready); /* signal all readers */
  else
    pthread_cond_signal(&lock->wready);	/* signal another writer */
  icmUnlockMutex(&lock->mutex);
}


