/***********************************************************************
(c) 1994-1999 Imperial College, F.G. McCabe  and Fujitsu Labs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Contact: fgm@fla.fujitsu.com
***********************************************************************/

/* 
 * Utility string functions for the ICM
 *
 */

#include "icmP.h"
#include "icmM.h"
#include "icmIoP.h"

#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

char *icmDataData(icmDataPo str)
{
  return str->data;
}

long icmDataLength(icmDataPo str)
{
  return str->size;
}

/* Convert a Data string record into a C-style string */
char *icmCstring(icmDataPo str,char *buff,long len)
{
  long i;
  for(i=0;i<str->size && --len>0;i++)
    buff[i]=str->data[i];

  buff[i]='\0';
  return buff;
}

char *icmDupStr(char *n)
{
  return strcpy(icmMalloc(strlen(n)+1),n);
}

void icmFreeStr(char *n)
{
  icmFree(n);
}

char *icmLowerStr(char *buffer,const char *orig,long count)
{
  register char *b = buffer;
  register const char *o = orig;

  while(count-->0 && (*b++=tolower(*o++))!='\0')
    ;
  *b = '\0';
  return buffer;

}

char *icmStrNCpy(char *dest,register char *s,long count)
{
  register char *d = dest;
  while(count-->0 && (*d++=*s++)!='\0')
    ;
  *d = '\0';			/* The system one doesnt do this ... stupidly */
  return dest;
}

char *icmStrNCat(char *dest,register char *s,long count)
{
  register char *d = dest;

  while(count-->0 && *d!='\0')
    d++;
  while(count-->0 && (*d++=*s++)!='\0')
    ;
  *d = '\0';			/* The system one doesnt do this ... stupidly */
  return dest;
}

char *icmStrExtract(char *dest,char *src,long count)
{
  int i;

  for(i=0;i<count;i++)
    dest[i]=src[i];
  dest[count]='\0';
  return dest;
}

/* Local version of sprintf for our fileing system */
typedef struct {
  icmTruth inUse;
  char *buffer;
  long len;
  long pos;
} StringRec;

#define OUTSTRS 8

static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
static StringRec outs[OUTSTRS];
static int nStrs = 0;

static icmStatus _flushStr(filePo f,int cl);
static icmStatus _closeStr(filePo f,int cl);
static icmStatus _refillString(filePo f,int cl);

filePo openInStr(char *buffer,long len)
{
  StringRec *info;
  int i;

  icmLockMutex(&mutex);

  for(i=0;i<nStrs && outs[i].inUse;i++)
    ;

  if(i==OUTSTRS){
    icmUnlockMutex(&mutex);
    return NULL;
  }

  info = &outs[i];
  if(i==nStrs)
    nStrs = i+1;

  info->buffer = buffer;
  info->len = len;
  info->pos = 0;
  info->inUse = icmYes;

  icmUnlockMutex(&mutex);

  return _new_io("<str>",ioString,ioREAD,i,_refillString,NULL,_closeStr,
		 NULL,NULL,NULL);
}

filePo openOutStr(char *buffer,long len)
{
  StringRec *info;
  int i;

  icmLockMutex(&mutex);

  for(i=0;i<nStrs && outs[i].inUse;i++)
    ;

  if(i==OUTSTRS){
    icmUnlockMutex(&mutex);
    return NULL;
  }

  info = &outs[i];
  if(i==nStrs)
    nStrs = i+1;

  info->buffer = buffer;
  info->len = len;
  info->inUse = icmYes;

  icmUnlockMutex(&mutex);

  return _new_io("<str>",ioString,ioWRITE,i,NULL,_flushStr,_closeStr,
		 NULL,NULL,NULL);
}

/* String file management */
static icmStatus _refillString(filePo f,int cl)
{
  StringRec* str = &outs[cl];
  int i;
  char *data = str->buffer;

  for(i=0;i<MAXLINE;i++)
    if(str->pos<str->len)
      f->in_line[i]=data[str->pos++];
    else
      break;

  f->in_len = i;
  f->in_pos = 0;
  if(i==0)
    f->status = icmEndFile;

  return f->status;
}
  
static icmStatus _flushStr(filePo f,int cl)
{
  StringRec *info = &outs[cl];
  int i;

  for(i=0;i<f->out_pos;i++){
    if(info->len>1){
      *info->buffer++ = f->out_line[i];
      info->len--;
    }
    else{
      strcpy(info->buffer-3,"...");
      break;
    }
  }
  f->out_pos = 0;
  return icmOk;
}

static icmStatus _closeStr(filePo f,int cl)
{
  StringRec *info = &outs[cl];

  *info->buffer = '\0';		/* Always terminate the output buffer */
  info->inUse = icmNo;
  if(info==&outs[nStrs-1]){
    icmLockMutex(&mutex);
    nStrs--;			/* Decrement number of outgoing strings */
    icmUnlockMutex(&mutex);
  }
  return icmOk;
}

/* external declaration of message handler */
icmStatus __voutMsg(filePo f,char *fmt,va_list args);

char *icmStrMsg(char *buffer,long len,char *fmt,...)
{
  filePo f = openOutStr(buffer,len);
  va_list args;			/* access the generic arguments */
  va_start(args,fmt);		/* start the variable argument sequence */

  __voutMsg(f,fmt,args);	/* Display into the string buffer */

  va_end(args);
  
  closeFile(f);			/* Close the string -- flushing into our buffer */
  return buffer;
}
