/***********************************************************************
(c) 1994-1999 Imperial College, F.G. McCabe  and Fujitsu Labs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Contact: fgm@fla.fujitsu.com
***********************************************************************/

/*
 * Access the system logging facilities
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <stdarg.h>
#include <sys/types.h>

#ifdef WIN32
#include <time.h>
#include <windows.h>
#include <io.h>
#else
#include <sys/time.h>
#include <sys/wait.h>
#include <unistd.h>
#include <limits.h>		/* Where OPEN_MAX is defined */
#include <syslog.h>
#endif

#ifdef SYSV
#include <stropts.h>
#endif

#include "icmP.h"
#include "icmIoP.h"		/* Get our own definitions */

#ifndef NumberOf
#define NumberOf(a) (sizeof(a)/sizeof(a[0]))
#endif

static filePo syslogF = NULL;

static icmStatus _flushLog(filePo f,int cl)
{
  char *cp = f->out_line;
  long len = f->out_pos;
  static char lineBuff[1024];	/* maximum size of log message we allow */
  static long pos=0;
  static pthread_mutex_t syslog_mutex = PTHREAD_MUTEX_INITIALIZER;

  /* First we have to copy into our line buffer */
  icmLockMutex(&syslog_mutex);

  while(len-->0){
    lineBuff[pos]=*cp++;

    if(lineBuff[pos]=='\n'){	/* flush current line out to syslog */
      lineBuff[pos]='\0';
      //syslog(LOG_NOTICE,"%s\n",lineBuff);
	  fprintf(stderr,"%s\n",lineBuff);
      pos = 0;
    }
    else if(pos<NumberOf(lineBuff)-1)
      pos++;
  }

  icmUnlockMutex(&syslog_mutex);

  return icmOk;
}

/* We can asssume that flush has already been called */
static icmStatus _closeLog(filePo f,int cl)
{
  //closelog();			/* wouldnt normally be used */
  return icmOk;
}

filePo icmSyslog(char *ident)
{
  //In windows we'll use sterr
  //openlog(ident,LOG_CONS,LOG_DAEMON);

  return syslogF = _new_io(ident,ioLog,ioWRITE,0,NULL,_flushLog,_closeLog,
			   NULL,NULL,NULL);
}

static pthread_mutex_t log_mutex;
static pthread_once_t logOnce = PTHREAD_ONCE_INIT;

static void initLog(void)
{
  icmCreateMutex(&log_mutex);
}

void logMsg(filePo out,char *fmt,...)
{
  if(out!=NULL){
    time_t now;
    va_list ap;

    pthread_once(&logOnce,initLog); 

    if(pthread_mutex_lock(&log_mutex)!=0)
      exit(errno);

    va_start(ap,fmt);

    if(out!=syslogF && time(&now)!=-1){
      struct tm *t = localtime(&now);
      char stamp[256];
      strftime(stamp,256,"%x %X",t);

      outMsg(out,"%s - ",stamp);
      __voutMsg(out,fmt,ap);
      outMsg(out,"\n");
    }
    else{
      __voutMsg(out,fmt,ap);
      outMsg(out,"\n");
    }
      
    va_end(ap);

    if(pthread_mutex_unlock(&log_mutex)!=0)
      exit(errno);
  }
  flushFile(out);
}

void icmLogMsg(char *fmt,...)
{
  if(icmLogFile!=NULL){
    time_t now;
    va_list ap;
    static pthread_mutex_t lg_mtx = PTHREAD_MUTEX_INITIALIZER;

    flushFile(icmLogFile);
    pthread_mutex_lock(&lg_mtx);

    va_start(ap,fmt);

    if(icmLogFile!=syslogF && time(&now)!=-1){
      struct tm *t = localtime(&now);
      char stamp[256];
      strftime(stamp,256,"%x %X",t);

      outMsg(icmLogFile,"%s - ",stamp);
      flushFile(icmLogFile);
      __voutMsg(icmLogFile,fmt,ap);
      outMsg(icmLogFile,"\n");
      flushFile(icmLogFile);
    }
    else{
      __voutMsg(icmLogFile,fmt,ap);
      outMsg(icmLogFile,"\n");
      flushFile(icmLogFile);
    }

    va_end(ap);
    pthread_mutex_unlock(&lg_mtx);
  }
  flushFile(icmLogFile);
}


