/*
 * This is a simple `echo' server implemented as an example to demonstrate 
 * The ICM API
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include "icm.h"
#include "icmFile.h"

char *name = "echo";
int portNo = 0;
#define LOGFILE "echo.log"
icmHandle ourHandle = NULL;
icmConn conn=NULL;
long delayTime = 0;			/* delay before sending replies */

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif
#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

void sig_int(int sig);
void sig_fatal(int sig);

static icmStatus HandleQuery(icmConn con,icmHandle recip,icmHandle snder,
			     icmOption opts,icmMsg msg,void *cl);

icmStatus pickOpt(char opt,char *optarg,void *cl)
{
  switch(opt){
  case 'P':
    portNo = atoi(optarg);
    return icmOk;
  case 'n':
    name = optarg;
    return icmOk;

  case 'd':
    delayTime = atoi(optarg);
    return icmOk;

  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	return icmError;
#endif
      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	return icmError;
      }
    return icmOk;
  }
  default:
    return icmError;
  }
}


int main(int argc, char **argv)
{
  if(icmGetOptions(argc,argv,"n:d:P:L:D:",pickOpt,NULL)==-1){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-n name] [-n delay] [-P port] [-L log]",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-n name] [-n delay] [-P port] [-L log]",argv[0]);
    exit(1);
  }

  if(icmLogFile == NULL)
    icmLogFile = newOutfile(LOGFILE); /* Default log file */

  ourHandle = icmParseHandle(name);

  if(icmInitComms(portNo,NULL,&conn)==icmOk){

    signal(SIGINT, sig_int);	/* set up termination routines */
    signal(SIGBUS, sig_fatal);
    signal(SIGSEGV, sig_fatal);
    signal(SIGFPE, sig_fatal);

    /* register ourselves */
    if(icmRegisterAgent(conn,ourHandle,NULL,&ourHandle)==icmOk){
      icmLogMsg("Echo server %h started",ourHandle);
      icmEventLoop(conn,HandleQuery,NULL);
      icmDeregisterAgent(conn,ourHandle);
    }
    else
      icmLogMsg("Failed to register %h",ourHandle);
  }
  else
    icmLogMsg("Failed to initialize ICM API");
  return 0;
}
	
static icmStatus HandleQuery(icmConn con,icmHandle recip,
			     icmHandle snder,icmOption opts,icmMsg msg,void *cl)
{
  icmHandle replyto = snder;

  icmIsOption(opts,icmReplyto,&replyto);

  icmLogMsg("Message `%M' from %h",msg,snder);

  if(icmScanMsg(msg,"%{quit%}")==icmOk){
    icmTerminateEventLoop(conn);
    icmFmtSendMsg(conn,replyto,ourHandle,NULL,"%{%(ok%,quit%)%}");
  }
  else{
    sleep(delayTime);
    icmFmtSendMsg(conn,replyto,ourHandle,NULL,"%{%(echo%#%)%}",msg);
  }
  return icmOk;
}

/*
 * cleanup after CTRL-C and other major errors
 */
void sig_int(int sig)
{
  icmLogMsg("Control-C exit");
  icmDeregisterAgent(conn,ourHandle);
  icmCloseComms(conn);
  exit(0);
}

void sig_fatal(int sig)
{
  icmLogMsg("bus error or segmentation fault");
  icmDeregisterAgent(conn,ourHandle);
  icmCloseComms(conn);
  exit(0);
}

