/*
 * This is a simple `message forwarded' server implemented as an example to demonstrate 
 * The ICM API
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "icm.h"
#include "icmFile.h"

char *name = "forward";
int portNo = 0;
#define LOGFILE "forward.log"
icmHandle ourHandle = NULL;
icmHandle forward = NULL;
icmConn conn=NULL;

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif
#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

void sig_int(int sig);
void sig_fatal(int sig);

static icmStatus HandleQuery(icmConn con,icmHandle recip,
			     icmHandle snder,icmOption opt,icmMsg msg,void *cl);

icmStatus pickOpt(char opt,char *optarg,void *cl)
{

  switch(opt){
  case 'P':
    portNo = atoi(optarg);
    return icmOk;
  case 'n':
    name = optarg;
    return icmOk;
  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	return icmError;
#endif
      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	return icmError;
      }
    return icmOk;
  }
  default:
    return icmError;
  }
}


int main(int argc, char **argv)
{
  int which;

  if((which=icmGetOptions(argc,argv,"n:P:L:D:",pickOpt,NULL))==-1 || which>=argc){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-n name] [-P port] [-L log] to",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-n name] [-P port] [-L log] to",argv[0]);
    exit(1);
  }

  if(icmLogFile == NULL)
    icmLogFile = newOutfile(LOGFILE); /* Default log file */

  ourHandle = icmParseHandle(name);
  forward = icmParseHandle(argv[which]);

  icmKeepHandle(ourHandle);
  icmKeepHandle(forward);

  if(icmInitComms(portNo,NULL,&conn)==icmOk){
    if(icmRegisterAgent(conn,ourHandle,(time_t)0,&ourHandle)==icmOk){
      signal(SIGINT, sig_int);	/* set up termination routines */
      signal(SIGBUS, sig_fatal);
      signal(SIGSEGV, sig_fatal);
      signal(SIGFPE, sig_fatal);

      icmLogMsg("Forwarding agent %h started",ourHandle);
      icmEventLoop(conn,HandleQuery,NULL);
      icmCloseComms(conn);
    }
    else
      icmLogMsg("Failed to register %h",ourHandle);
  }
  else
    icmLogMsg("Failed to initialize ICM API");
  return 1;
}
	
static icmStatus HandleQuery(icmConn con,icmHandle recip,
			     icmHandle snder,icmOption opts,icmMsg msg,void *cl)
{
  icmHandle replyto=snder;

  icmIsOption(opts,icmReplyto,&replyto);
  icmLogMsg("Message `%M' from %h to %h",msg,snder,recip);

  if(icmScanMsg(msg,"quit")==icmOk){
    icmMsg replyMsg = icmFormatMsg(NULL,"%(ok%,quit%)");

    icmTerminateEventLoop(conn);
    icmSendMsg(conn,replyto,recip,NULL,replyMsg);
    icmReleaseMsg(replyMsg);
  }
  else{
    icmOption opts = icmNewOpt(NULL,icmReplyto,replyto);
    icmSendMsg(conn,forward,ourHandle,opts,msg);
    icmReleaseOptions(opts);
  }
  return icmOk;
}

/*
 * cleanup after CTRL-C and other major errors
 */
void sig_int(int sig)
{
  icmLogMsg("Control-C exit");
  icmCloseComms(conn);
  exit(0);
}

void sig_fatal(int sig)
{
  icmLogMsg("bus error or segmentation fault");
  icmCloseComms(conn);
  exit(0);
}

