/*
 * This is a simple file injector implemented as an example to demonstrate 
 * The ICM API
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "icm.h"
#include "icmFile.h"

char *name = "inject";
char *file = NULL;

int portNo = 0;
#define LOGFILE "inject.log"
icmHandle ourHandle = NULL;
icmConn conn=NULL;

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif
#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

void sig_int(int sig);
void sig_fatal(int sig);

static icmMsg makeFileMsg(char *file);

icmStatus pickOpt(char opt,char *optarg,void *cl)
{
  int *errflg = (int*)cl;

  switch(opt){
  case 'P':
    portNo = atoi(optarg);
    return icmOk;

  case 'n':
    name = optarg;
    return icmOk;

  case 'f':
    file = optarg;
    return icmOk;

  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	(*errflg)++;
	return icmError;
#endif
      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	(*errflg)++;
	return icmError;
      }
    return icmOk;
  }
  default:
    (*errflg)++;
    return icmError;
  }
}


int main(int argc, char **argv)
{
  int errflag = 0;
  int which;
  icmHandle tgt;

  if((which=icmGetOptions(argc,argv,"f:n:P:L:D:",pickOpt,&errflag))==-1 ||
     which>=argc){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-f file] [-n name] [-P port] [-L log] to",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-f file] [-n name] [-P port] [-L log] to",argv[0]);
    exit(1);
  }

  tgt = icmParseHandle(argv[which]);

  if(icmLogFile == NULL)
    icmLogFile = newOutfile(LOGFILE); /* Default log file */

  ourHandle = icmParseHandle(name);

  if(icmInitComms(portNo,NULL,&conn)==icmOk){
    icmMsg msg = makeFileMsg(file);

    if(msg!=NULL){
      /* register ourselves */
      if(icmRegisterAgent(conn,ourHandle,(time_t)0,&ourHandle)==icmOk){
	icmSendMsg(conn,tgt,ourHandle,NULL,msg);
	icmReleaseMsg(msg);

	icmCloseComms(conn);	/* Weve injected the file message ... */
      }
      else{
	icmLogMsg("Couldnt register %h",ourHandle);
	exit(-1);
      }
    }
    else{
      icmLogMsg("Couldnt create file message %s",file);
      exit(-1);
    }
  }
  else
    icmLogMsg("Failed to initialize ICM API");
  return 1;			/* We will never get here */
}
	
static icmMsg makeFileMsg(char *file)
{
  filePo in = (file==NULL?OpenStdin():openInFile(file));

  if(in!=NULL){
    icmDataRec str;
    icmMsg msg;
    int ch;
    char *buffer=(char*)malloc(1024);
    long max = 1024;
    long pos = 0;

    while((ch=inCh(in))!=EOF){
      if(pos>=max){
	long newMax = max+(max>>1);
	char *newBuff = (char*)realloc(buffer,newMax);

	if(newBuff==NULL){
	  icmLogMsg("Cant grow file buffer to %d bytes",newMax);
	  exit(1);
	}

	buffer = newBuff;
	max = newMax;
      }
      buffer[pos++] = ch;
    }

    str.data = buffer;
    str.size = pos;

    msg = icmFormatMsg(NULL,"%(file%s%S%)",file,&str);

    free(buffer);
    return msg;
  }
  else
    return NULL;
}





