/*
 * This asks a commserver to list agents
 * implemented as an example to demonstrate 
 * The ICM API
 */

#include "icm.h"
#include "icmP.h"
#include "icmFile.h"
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <limits.h>

char *name = "list";
char *file = NULL;

int portNo = 0;
icmHandle ourHandle = NULL;
icmConn conn=NULL;

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif
#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

icmStatus pickOpt(char opt,char *optarg,void *cl)
{
  int *errflg = (int*)cl;

  switch(opt){
  case 'P':
    portNo = atoi(optarg);
    return icmOk;

  case 'n':
    name = optarg;
    return icmOk;

  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	(*errflg)++;
	return icmError;
#endif
      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	(*errflg)++;
	return icmError;
      }
    return icmOk;
  }
  default:
    (*errflg)++;
    return icmError;
  }
}


int main(int argc, char **argv)
{
  int errflag = 0;
  int which;

  if((which=icmGetOptions(argc,argv,"n:P:L:D:",pickOpt,&errflag))==-1){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-n name] [-P port] [-L log] [cs]",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-n name] [-P port] [-L log] [cs]",argv[0]);
    exit(1);
  }

  if(icmLogFile == NULL)
    icmLogFile = OpenStdout();	/* Default is output */

  ourHandle = icmParseHandle(name);

  if(icmInitComms(portNo,NULL,&conn)==icmOk){
    if(icmRegisterAgent(conn,ourHandle,(time_t)0,&ourHandle)==icmOk){
      icmHandle list[256];
      long count = 256;
      char *host = which<argc?argv[which]:icmMachineName();
      icmHandle tgt = icmCommServerHandle(host);
      icmMsg msg = icmFormatMsg(NULL,"icmListAgents");
      icmStatus ret = icmSendMsg(conn,tgt,ourHandle,NULL,msg);
      long seqNo = -LONG_MAX;
      icmHandle to,sndr;
      icmOption opts;

      while((ret=icmGetMsg(conn,&to,&sndr,&opts,&msg,&seqNo))==icmOk){
	if(icmSameAgentHandle(sndr,tgt)==icmOk &&
	   icmSameAgentHandle(to,ourHandle)==icmOk){
	  if(icmScanMsg(msg,"%(icmListAgents%,icmFailed%)")==icmOk){
	    icmReleaseMsg(msg);	/* release this message */
	    icmReleaseOptions(opts);
	    icmLogMsg("Failed to get list from %h",tgt);
	    break;
	  }
	  else if(icmScanMsg(msg,"%(icmListAgents%*h%)",&count,list)==icmOk){
	    int i;
	    
	    icmLogMsg("Agents registered at %s:",host);
	    for(i=0;i<count;i++)
	      outMsg(icmLogFile,"      " BOLD_ESC_ON "%h" BOLD_ESC_OFF "\n",list[i]);
	    icmReleaseMsg(msg);	/* release this message */
	    icmReleaseOptions(opts);
	    break;
	  }
	  else
	    icmReplaceMsg(conn,to,sndr,opts,msg,seqNo);
	}
	else
	  icmReplaceMsg(conn,to,sndr,opts,msg,seqNo);
      }
      icmDeregisterAgent(conn,ourHandle);
      icmCloseComms(conn);
      return 0;
    }
    else
      icmLogMsg("Failed to register %h",ourHandle);
  }
  else
    icmLogMsg("Failed to initialize ICM API");
  return 1;			/* We should never get here */
}
	
