/*
 * This is a simple `agent pinger' implemented as an example to demonstrate 
 * the ICM API.
 * It queries the local commserver for the status of the agent
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <signal.h>
#include "icm.h"
#include "icmFile.h"

char *name = "ping";

int portNo = 0;
#define LOGFILE "ping.log"
icmHandle ourHandle = NULL;
icmConn conn=NULL;

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif
#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

void sig_int(int sig);
void sig_fatal(int sig);

icmStatus pickOpt(char opt,char *optarg,void *cl)
{
  int *errflg = (int*)cl;

  switch(opt){
  case 'P':
    portNo = atoi(optarg);
    return icmOk;

  case 'n':
    name = optarg;
    return icmOk;

  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	(*errflg)++;
	return icmError;
#endif
      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	(*errflg)++;
	return icmError;
      }
    return icmOk;
  }
  default:
    (*errflg)++;
    return icmError;
  }
}

int main(int argc, char **argv)
{
  int errflag = 0;
  int which;
  icmHandle tgt;

  if((which=icmGetOptions(argc,argv,"n:P:L:D:",pickOpt,&errflag))==-1 ||
     which>=argc){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-n name] [-P port] [-L log] to",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-n name] [-P port] [-L log] to\n",argv[0]);
    exit(1);
  }

  tgt = icmParseHandle(argv[which]);

  if(icmLogFile == NULL)
    icmLogFile = newOutfile(LOGFILE); /* Default log file */

  ourHandle = icmParseHandle(name);

  if(icmInitComms(portNo,NULL,&conn)==icmOk){
    if(icmRegisterAgent(conn,ourHandle,(time_t)0,&ourHandle)==icmOk){
      switch(icmPingAgent(conn,ourHandle,tgt,&tgt)){
      case icmOk:
	icmLogMsg("%h agent alive and well",tgt);
	return 0;
      case icmFailed:
	icmLogMsg("%h agent unavailable",tgt);
	return 0;
      default:
	icmLogMsg("ping problem",tgt);
	return 1;
      }
    }
    else
      icmLogMsg("Failed to register with local commserver");

  }
  icmLogMsg("Failed to initialize ICM API");
  return 1;
}
	




