/*
 * This is a simple `agent pinger' implemented as an example to demonstrate 
 * the ICM API
 * It sends a message to an named agent and either immediately detaches itself
 * or it waits for a reply.
 * It can be restarted in attach-mode which doesnt involve sending the initial msg
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <signal.h>
#include <string.h>
#include "icm.h"
#include "icmFile.h"

char *name = "send";
char *msgText = "quit";

int portNo = 0;
#define LOGFILE "send.log"
icmHandle ourHandle = NULL;
icmConn conn=NULL;
enum {sendNwait, sendNdetach, attachNwait } mode = sendNwait;

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif
#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

void sig_int(int sig);
void sig_fatal(int sig);

icmStatus pickOpt(char opt,char *optarg,void *cl)
{
  int *errflg = (int*)cl;

  switch(opt){
  case 'P':
    portNo = atoi(optarg);
    return icmOk;

  case 'n':
    name = optarg;
    return icmOk;

  case 'k':
    msgText = optarg;
    return icmOk;

  case 'm':
    if(strcmp(optarg,"sendNwait")==0)
      mode = sendNwait;
    else if(strcmp(optarg,"sendNdetach")==0)
      mode = sendNdetach;
    else if(strcmp(optarg,"attachNwait")==0)
      mode = attachNwait;
    else
      return icmError;
    return icmOk;

  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	(*errflg)++;
	return icmError;
#endif
      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	(*errflg)++;
	return icmError;
      }
    return icmOk;
  }
  default:
    (*errflg)++;
    return icmError;
  }
}


int main(int argc, char **argv)
{
  int errflag = 0;
  int which;
  icmHandle tgt;

  if((which=icmGetOptions(argc,argv,"k:n:P:L:D:m:",pickOpt,&errflag))==-1 ||
     which>=argc){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-k msg] [-n name] [-m mode] [-P port] [-L log] to",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-k msg] [-n name] [-m mode] [-P port] [-L log] to\n",argv[0]);
    exit(1);
  }

  tgt = icmParseHandle(argv[which]);

  if(icmLogFile == NULL)
    icmLogFile = newOutfile(LOGFILE); /* Default log file */

  ourHandle = icmParseHandle(name);

  if(icmInitComms(portNo,NULL,&conn)==icmOk){
    switch(mode){
    case sendNdetach:{
      if(icmRegisterAgent(conn,ourHandle,(time_t)0,&ourHandle)==icmOk){
	icmMsg msg = icmFormatMsg(NULL,"%s",msgText);

	icmSendMsg(conn,tgt,ourHandle,NULL,msg);

	icmDetachAgent(conn,ourHandle,NULL);
	icmLogMsg("%M sent and we've detached",msg);
	icmReleaseMsg(msg);
      }
      else
	icmLogMsg("Couldnt register %h",ourHandle);
      break;
    }

    case sendNwait:{
      if(icmRegisterAgent(conn,ourHandle,(time_t)0,&ourHandle)==icmOk){
	icmMsg msg = icmFormatMsg(NULL,"%s",msgText);
	icmHandle to,sndr;
	long seqNo = -LONG_MAX;
	icmOption opts;

	icmSendMsg(conn,tgt,ourHandle,NULL,msg);

	icmLogMsg("%M sent",msg);

	icmReleaseMsg(msg);

	switch(icmGetMsg(conn,&to,&sndr,&opts,&msg,&seqNo)){
	case icmOk:
	  icmLogMsg("We have a reply from %h: %M",sndr,msg);
	  break;
	default:
	  icmLogMsg("Problem in getting reply message");
	}
	icmReleaseOptions(opts);
      }
      else
	icmLogMsg("Couldnt register %h",ourHandle);
      break;
    }

    case attachNwait:{
      if(icmAttachAgent(conn,ourHandle,NULL)==icmOk){
	icmHandle to,sndr;
	icmOption opts;
	long seqNo = -LONG_MAX;
	icmMsg msg;

	icmLogMsg("Attached and waiting for reply");

	switch(icmGetMsg(conn,&to,&sndr,&opts,&msg,&seqNo)){
	case icmOk:
	  icmLogMsg("We have a reply from %h: %M",sndr,msg);
	  break;
	default:
	  icmLogMsg("Problem in getting reply message");
	}
      }
      else
	icmLogMsg("Couldnt register %h",ourHandle);
    }
    }
    icmDeregisterAgent(conn,ourHandle);
    icmCloseComms(conn);
    return 0;
  }
  else{
    icmLogMsg("Failed to initialize ICM API");
    return 1;
  }
}
	




