/*
 * This is a simple `agent monitor' implemented as an example to demonstrate 
 * the ICM API
 * It issues a request of the commserver to monitor another agent.
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <signal.h>
#include <string.h>
#include "icm.h"
#include "icmFile.h"

char *name = "wait";
char *msgText = "quit";

int portNo = 0;
#define LOGFILE "wait.log"
icmHandle ourHandle = NULL;
icmConn conn=NULL;

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif
#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

void sig_int(int sig);
void sig_fatal(int sig);

static icmStatus HandleResponse(icmConn con,icmHandle recip,icmHandle snder,
				icmOption opts,icmMsg msg,void *cl);

icmStatus pickOpt(char opt,char *optarg,void *cl)
{
  int *errflg = (int*)cl;

  switch(opt){
  case 'P':
    portNo = atoi(optarg);
    return icmOk;

  case 'n':
    name = optarg;
    return icmOk;

  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	(*errflg)++;
	return icmError;
#endif
      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	(*errflg)++;
	return icmError;
      }
    return icmOk;
  }
  default:
    (*errflg)++;
    return icmError;
  }
}


int main(int argc, char **argv)
{
  int errflag = 0;
  int which;
  icmHandle tgt;

  if((which=icmGetOptions(argc,argv,"n:P:L:D:",pickOpt,&errflag))==-1 ||
     which>=argc){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-n name] [-P port] [-L log] agent",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-n name] [-P port] [-L log] agent\n",argv[0]);
    exit(1);
  }

  tgt = icmParseHandle(argv[which]);

  if(icmLogFile == NULL)
    icmLogFile = newOutfile(LOGFILE); /* Default log file */

  ourHandle = icmParseHandle(name);

  if(icmInitComms(portNo,NULL,&conn)==icmOk){
    if(icmRegisterAgent(conn,ourHandle,(time_t)0,&ourHandle)==icmOk){
      if(icmMonitorAgent(conn,ourHandle,tgt)==icmOk){
	icmLogMsg("%h registered and waiting for %h",ourHandle,tgt);
	icmEventLoop(conn,HandleResponse,tgt);
      }
      else
	icmLogMsg("Couldnt monitor %h",tgt);
    }
    else
      icmLogMsg("Couldnt register %h",ourHandle);
    icmCloseComms(conn);
    return 0;
  }
  else{
    icmLogMsg("Failed to initialize ICM API");
    return 1;
  }
}
	
static icmStatus HandleResponse(icmConn con,icmHandle recip,icmHandle snder,
				icmOption opts,icmMsg msg,void *cl)
{
  char rep[1024];
  icmHandle h;

  icmLogMsg("Message `%M' from %h to %h",msg,snder,recip);

  if(icmScanMsg(msg,"quit")==icmOk)
    icmTerminateEventLoop(conn);
  else if(icmScanMsg(msg,"%(icmMonitorAgent%s%h%)",rep,&h)==icmOk){
    if(strcmp(rep,"icmRegisterAgent")==0)
      icmLogMsg("%h registered",h);
    else if(strcmp(rep,"icmDeregisterAgent")==0)
      icmLogMsg("%h terminated",h);
    else if(strcmp(rep,"icmDetachAgent")==0)
      icmLogMsg("%h detached",h);
    else if(strcmp(rep,"icmAttachAgent")==0)
      icmLogMsg("%h attached",h);
    else
      icmLogMsg("Unknown message: %M",msg);
  }
  else
    icmLogMsg("Unknown message: %M",msg);
  return icmOk;
}


