#ifndef _DAEMON_H_
#define _DAEMON_H_

/* Header file for the April communications daemon */
/* select system header files */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "icmP.h"		/* ICM header file */
#include "icmM.h"
#include "icmIoP.h"
#include "icmW.h"
#include "icmFile.h"
#include "icmSock.h"

#ifdef WIN32
#include <winsock.h>
#include <windows.h>
#include <io.h>
#include <time.h>
#else
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#endif

#include <limits.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>

#ifndef MAXNAME
#define MAXNAME 512		/* Maximum size of a file name of host name */
#endif

#ifndef LOGFILE
#define LOGFILE "icm-log.txt" /* Log file */
#endif

#ifndef STAYUPTIME
#define STAYUPTIME 600		/* 10 minutes before closing a remote connection */
#endif

typedef enum {error, failed, partial, okay} status;

extern icmHandle ourHandle;	/* Our name in this world */
extern int ourPort;		/* Our TCP port number -- allow multiple icms */

void exit_daemon(int code);
status MakeError(char *reason);
extern char *error_code;
long cmp_name(void *a, void *b);
unsigned long hash_name(void *n);

typedef struct _post_box_ *PostPo;
typedef struct _agent_name_ *agentNamePo;
typedef struct _agent_rec_ *agentPo;

status initPostOffice(icmHandle ourHandle);
void closePostOffice(void);
void dumpPostOffice();

status initBoxes(int count);
void closeBoxes(void);
PostPo OpenPostBox(icmIpPort addr,icmConn conn,char *host,time_t timeout);
status PostMsg(PostPo box,icmHandle recip,icmHandle sender,icmOption opts,icmMsg m);
void refBox(PostPo box);
void unRefBox(PostPo box);
void clearOSMessages(agentPo agent,PostPo box);
void tryRedelivery(void);
void dumpBoxes(void);
icmHandle mergeLocations(icmHandle h1,icmHandle h2);
icmHandle removeLocation(icmHandle h,char *duff);
icmTruth isOurAddress(char *loc);
status isOurHandle(icmHandle h);

status registerAgent(icmHandle handle,icmHandle *real,time_t lease,PostPo box);
status deregisterAgent(icmHandle handle,PostPo box);
status registeredAgent(icmHandle handle,icmHandle *real);
void deregisterAllOnBox(PostPo box);
status attachAgent(icmHandle handle,time_t lease,PostPo box);
status detachAgent(icmHandle handle,PostPo box);
PostPo agentBox(icmHandle handle);
icmHandle agentAddress(icmHandle handle);
icmStatus agentRegistered(icmHandle handle,icmHandle *real);
agentPo agentOf(icmHandle handle);
icmHandle *listAllAgents(long *count);
void freeAgentList(icmHandle *list,long count);
status monitorAgent(icmHandle ag,icmHandle client);
status unmonitorAgent(icmHandle ag,icmHandle client);
void reportAgentStatus(agentPo agent,icmHandle client,PostPo box);
long detachAllOnBox(PostPo box);
void reattachAllOnBox(PostPo box,long tag);

typedef status (*serverProc)(icmHandle recip,icmMsg msg,PostPo box);

void initServer(void);
filePo acceptSecuredConnection(filePo listen,icmStatus *res);
void ServerMsg(icmHandle recip,icmHandle sndr,icmOption opts,icmMsg msg,PostPo box);
void processMsg(icmHandle recip,icmHandle snder,icmOption opts,icmMsg msg,PostPo box);

status initForwarding(long count);
status requestForwarding(icmHandle handle,icmHandle forward);
status cancelForwarding(icmHandle handle,icmHandle forward);
void forwardMessage(icmHandle recip,icmHandle snder,icmOption opts,icmMsg msg);
void handleFwdFailed(icmDataPo badLoc,icmHandle reply,icmHandle snder,
		     icmOption opts,icmMsg msg);
void forwardSvrRequest(icmHandle recip,icmHandle snder,icmOption opts,icmMsg msg);
PostPo fwdBox(icmHandle recip);
icmConn connectToRemote(char *loc);

void setupsignals(void);
void blockSignals(void);

#define DISPLAYNAME "<display>" /* Name of our special display agent */

#ifdef ALLTRACE
#ifndef COMMTRACE
#define COMMTRACE
#endif

#ifndef SERVERTRACE
#define SERVERTRACE
#endif

#ifndef MUTEXTRACE
#define MUTEXTRACE
#endif

#endif

#ifdef COMMTRACE
extern icmTruth traceMsg;
#endif

#ifdef SERVERTRACE
extern icmTruth traceSvr;	/* True if we are tracing the server's funcs */
#endif

#ifdef MUTEXTRACE
extern icmTruth traceMutex;	/* True if we are tracing mutexes */
#endif

#if DMALLOC
#include <dmalloc.h>		/* debugging malloc library */
#endif

#endif
