/*
 * Functions to check the /etc/hosts.allow and /etc/hosts.deny files
 * to see if a particular connection will be permitted.
 */
#include "config.h"		/* Invoke configuration header */
#include "daemon.h"

#define MAXLINE 512

static int hostMatches(char *pattern,char *host);

icmStatus checkForHost(char *host,char *demon,char *fileName)
{
  filePo f = openInFile(fileName);

  if(f==NULL)
    return icmFailed;
  else{
    char lineBuffer[MAXLINE];

    while(fileAtEof(f)!=icmEndFile){
      switch(inLine(f,lineBuffer,MAXLINE,"\n")){
      case icmOk:{
	char *l=lineBuffer;

	if(strlen(lineBuffer)==0 || lineBuffer[0]=='#')
	  continue;

	/* process the list of daemons */
	while(1){
	  char daemon[MAXLINE], *d=daemon;

	  while(*l!='\0' && strchr(*l," \t")!=NULL)
	    l++;		/* skip spaces */

	  while(*l!='\0' && strchr(*l," \t:")==NULL)
	    *d++=*l++;
	  *d='\0';

	  if(strcmp(daemon,"ALL")==0 ||
	     strcmp(daemon,"all")==0 ||
	     strcmp(daemon,demon)==0){ /* We found the correct daemon entry */
	    while(*l!='\0' && strchr(*l,":")==NULL)
	      l++;

	    while(1){		/* process the list of host names */
	      char hst[MAXLINE], *h=hst;

	      while(*l!='\0' && strchr(*l," \t")!=NULL)
		l++;		/* skip spaces to the next host name */

	      while(*l!='\0' && strchr(*l," \t:")==NULL)
		*h++=*l++;	/* collect the host name/pattern */
	      *h='\0';

	      if(hostMatches(h,host))
		return icmOk;
	    }
	  }
	  while(*l!='\0' && strchr(*l," \t")!=NULL)
	    l++;

	  if(*l==':' || *l=='\0')
	    break;		/* This line doesnt work ... */
	  else
	    continue;
	}
      }
      case icmEndFile:
	return icmFailed;
      default:
	return icmError;
      }
    }
  }
}

static int hostMatches(char *pattern,char *host)
{
  char ptn[MAXLINE];
  char hst[MAXLINE];

  icmLowerStr(ptn,pattern,MAXLINE);
  icmLowerStr(hst,host,MAXLINE);

  if(strcmp(ptn,"all")==0 ||strcmp(ptn,host)==0)
  {
    return 1;
  }
  else if(strcmp(ptn,"local")==0)
  {
    return strchr(hst,".")==NULL;
  }
  else {
    return 0;
  }
}
