/***********************************************************************
(c) 1994-1999 F.G. McCabe, Imperial College and Fujitsu Labs

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Contact: fgm@fla.fujitsu.com
***********************************************************************/

#ifndef _ICM_IO_P_H_
#define _ICM_IO_P_H_

#include <sys/types.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <fcntl.h>
#ifdef WIN32
#include <io.h>
#else
#include <unistd.h>
#endif
#include <pthread.h>		/* We are supposed to be thread-safe */
#include "icmFile.h"

typedef enum {ioNULL=000,ioREAD=001,ioWRITE=002} ioState;

/* the internal I/O descriptor table */
/*
 * This defines our I/O descriptors.
 */
#ifndef MAXLINE
#define MAXLINE 1024		/* Size of a standard buffer */
#endif

#ifndef MAXFILELEN
#define MAXFILELEN 256		/* Maximum length of a file name */
#endif

#ifndef IOMAX
#define IOMAX	FD_SETSIZE
#endif

/* IO management procedures */
typedef icmStatus (*fileConfProc)(filePo f,icmIoConfigOpt mode,int cl);
typedef icmStatus (*fileMsgProc)(filePo f,void *data,int width,int precision,icmTruth alt);

typedef struct _file_rec_{
  char filename[MAXFILELEN];	/* File name */
  ioState mode;			/* Mode that file is opened for */
  ioType type;			/* Type of file or device */

  unsigned char *in_line;	/* Input Line buffer */
  int in_pos;			/* Where in the current line are we? */
  int in_len;			/* How many characters in the current line? */
  int line_number;		/* How many lines have we read */
  long f_pos;			/* File position in file */
  icmStatus status;		/* Current file status */

  char *out_line;		/* Output line buffer */
  int out_pos;			/* Current output position */
  int depth;			/* Printing depth */

  int client;			/* Special client data */
  void *client2;		/* 2nd special client data */
  fileProc refill;		/* Called when file buffer is to be refilled */
  fileProc flusher;		/* Called when a file is to be flushed */
  fileProc closer;		/* Called when a file is to be closed */
  fileProc readyin;		/* Called to determine if file has input */
  fileProc readyout;		/* Called to determine if file can output */
  fileConfProc configure;	/* Called when changing file mode */
  pthread_mutex_t i_mutex;	/* Mutex used for input channel */
  pthread_mutex_t o_mutex;	/* Mutex used for output channel */
  long refCount;		/* Reference count */
} IoEntry;

#ifdef WIN32
#define STD_PERMISSIONS
#else
#define STD_PERMISSIONS (S_IXUSR|S_IXGRP|S_IXOTH|S_IRUSR|S_IRGRP|S_IROTH|S_IWUSR)
#endif

filePo _new_io(char *name,ioType type,int mode,int client,
	       fileProc refill,fileProc flusher,fileProc clser,
	       fileProc readyin,fileProc readyout,fileConfProc configure);
icmStatus _fileInReady(filePo f,int cl);
icmStatus _fileOutReady(filePo f,int cl);
void _initMsgProcs_(void);
void _installMsgProc(char key,fileMsgProc proc);
icmStatus __voutMsg(filePo f,char *fmt,va_list args);

long icmIoRefCount(filePo);
void icmIncRefCount(filePo);
void icmDecRefCount(filePo);

void triggerIo(fileProc filter,ioType type,int cl);
filePo findIo(ioType type,int cl);
filePo openOutStr(char *buffer,long len);
filePo openInStr(char *buffer,long len);

ioState fileMode(filePo f);
#endif

