/*
 * This is the communications daemon for the ICM
 * (c) 1994-1998 Imperial College and F.G. McCabe
 * All rights reserved
 *
 */

#include "config.h"		/* Invoke configuration header */
#include "daemon.h"		/* Our data structures etc. */
#include "version.h"		/* Version information */

char *error_code = NULL;	/* Error message string */

#ifndef MAXPROXY
#define MAXPROXY 10
#endif

char *our_name = ICM_COMMSERVER; /* What is our name? */
int ourPort = ICM_COMMS_PORT;	/* Standard port number */
char ourHost[MAXSTRING];	/* what is our host name? */
char ourIP[MAXPROXY][MAXSTRING]; /* what are our IP's */
static char *our_locs[MAXPROXY];
static int top_loc = 0;

filePo listener=NULL;		/* Listening file */
icmHandle ourHandle = NULL;	/* Our name in this world */
static int daemon_init(void);

static icmStatus pickOption(char opt,char *optarg,void *client)
{
  switch(opt){
  case 'D':{			/* Debug options */
    char *d = optarg;
    while(*d!='\0')
      switch(*d++){

      case 'm':			/* Tracing TCP-level messages? */
#ifdef COMMTRACE
	traceMsg = icmYes;
	continue;
#else
	fprintf(stderr,"Message tracing not enabled\n");
	return icmError;
#endif

      case 's':			/* Tracing server functions? */
#ifdef SERVERTRACE
	traceSvr = icmYes;
	continue;
#else
	fprintf(stderr,"Server tracing not enabled\n");
	return icmError;
#endif

      case 'M':			/* Tracing monitoring? */
#ifdef MONITORTRACE
	traceMonitor = icmYes;
	continue;
#else
	fprintf(stderr,"Monitor tracing not enabled\n");
	return icmError;
#endif
	continue;

      case 'h':			/* Tracing handle mgt? */
#ifdef HANDLETRACE
	traceHandle = icmYes;
	continue;
#else
	fprintf(stderr,"Handle tracing not enabled\n");
	return icmError;
#endif

      case 'w':			/* Tracing queue worker */
#ifdef WKRTRACE
	traceWkr = icmYes;
	continue;
#else
	fprintf(stderr,"Worker Q tracing not enabled\n");
	return icmError;
#endif

      case 'x':			/* Tracing mutex messages? */
#ifdef MUTEXTRACE
	traceMutex = icmYes;
	continue;
#else
	fprintf(stderr,"Mutex tracing not enabled\n");
	return icmError;
#endif

      case 'H':			/* Tracing memory */
#ifdef MEMTRACE
	traceMem = icmYes;
	continue;
#else
	fprintf(stderr,"Memory tracing not enabled\n");
	return icmError;
#endif

      case '*':			/* trace everything */
#ifdef COMMTRACE
	traceMsg = icmYes;
#endif
#ifdef SERVERTRACE
	traceSvr = icmYes;
#endif
#ifdef MONITORTRACE
	traceMonitor = icmYes;
#endif
#ifdef HANDLETRACE
	traceHandle = icmYes;
#endif
#ifdef WKRTRACE
	traceWkr = icmYes;
#endif
#ifdef MUTEXTRACE
	traceMutex = icmYes;
#endif
#ifdef MEMTRACE
	traceMem = icmYes;
#endif
	continue;

      default:
	fprintf(stderr,"Unknown debug option [%c]\n",*(d-1));
	return icmError;
      }
    return icmOk;
  }
  case 'P':
    ourPort = atoi(optarg);
    return icmOk;

  case 'l':{			/* attach an alternate location to the ICM */
    if(top_loc<MAXPROXY)
      our_locs[top_loc++]=optarg;

    return icmOk;
  }

  case 'n':
    our_name = optarg;
    return icmOk;

  default:
    return icmError;
  }
}

/*
 * main program
 */
int main(int argc, char **argv)
{

  //Variables come first in C!
  int i;

  //Let's get this done FIRST
  int wsockErr;
  WSADATA wsaData;

  //Init winsock
  WORD wVersionRequested = MAKEWORD( 2, 2 );
  wsockErr = WSAStartup(wVersionRequested, &wsaData );


  
  initICMIo();

  our_locs[0] = icmMachineIP();	/* set up default location */
  top_loc = 1;

  strncpy(ourHost,icmMachineName(),NumberOf(ourHost)-1);

  i = 0;
  while(i<NumberOf(ourIP)-1 && 
	icmGetNthHostIP(ourHost,i,&ourIP[i][0],NumberOf(ourIP[i])-1)!=NULL)
    i++;
  for(;i<NumberOf(ourIP);i++)
    ourIP[i][0]='\0';

  if(icmGetOptions(argc,argv,"P:Q:p:n:l:D:A:",pickOption,NULL)==-1){
    if(icmLogFile!=NULL)
      icmLogMsg("Usage: %s [-n name] [-P port] [-l location]* [-L log]",argv[0]);
    else
      fprintf(stderr,"Usage: %s [-n name] [-P port] [-l location]* [-L log]",argv[0]);
    exit(1);
  }

  if(icmLogFile == NULL)
    if(icmInitLogfile(LOGFILE)!=icmOk){
      fprintf(stderr,"%s: failed to open " LOGFILE "\n",argv[0]);
      exit(1);
    }

  ourHandle = icmMakeHandle(NULL,our_name,icmMachineName(),top_loc,our_locs);
  icmKeepHandle(ourHandle);

  if(icmLogFile!=stdOut)
    daemon_init();

  initPostOffice(ourHandle);

  for(i=1;i<top_loc;i++)
    connectToRemote(our_locs[i]); /* connect to remote commservers */

  listener = listeningPort(our_name,ourPort);

  if(listener==NULL){
    icmLogMsg("Failed to set up commserver");
    exit(1);
  }

  setupsignals();

  icmLogMsg("starting %s as %h on port %d",version,ourHandle,ourPort);

  while(icmYes){
    icmStatus res;
    filePo remote = acceptSecuredConnection(listener,&res);

    switch(res){
    case icmOk:{
      int cl_port;
      char *cl_name = peerName(remote,&cl_port);
      icmConn conn = icmMakeConn(remote,cl_port,cl_name);
      icmIpPort addr = icmPeerAddr(remote);

      OpenPostBox(addr,conn,cl_name,-1);
 
      icmLogMsg("New connection: %Q",conn);

      continue;
    }
    case icmFailed:
      break;
    case icmError:
      icmLogMsg("problem with accepting a new connection");
      return 0;
    case icmEndFile:{
      icmLogMsg("closing down commserver");
      return 0;
    }
    }
  }
  return 0;			/* We arent ever going to get here ... */
}

icmTruth isOurAddress(char *loc)
{
  int i;

  if(strcmp(loc,ourHost)==0)
    return icmYes;
  for(i=0;i<NumberOf(ourIP)-1;i++)
    if(strcmp(ourIP[i],loc)==0)
      return icmYes;
  return icmNo;
}

status isOurHandle(icmHandle h)
{
  char *tgt,*name,*home;

  if(icmAnalyseHandle(h,&tgt,&name,&home,NULL,NULL)==icmOk){
    if((tgt==NULL || strcmp(tgt,"")==0) && strcmp(name,our_name)==0){
      if(strcmp(home,icmMachineName())==0)
	return okay;
      else{
	int i = 0;
	while(i<NumberOf(ourIP)){
	  if(strcmp(home,ourIP[i])==0)
	    return okay;
	  else
	    i++;
	}
	return failed;
      }
    }
    else
      return failed;
  }
  else
    return error;
}

status MakeError(char *reason)
{
  icmLogMsg("Error: " BOLD_ESC_ON "%s" BOLD_ESC_OFF,reason);
  error_code = reason;
  return error;
}

static int daemon_init(void)
{
#ifndef WIN32
  pid_t pid = fork();

  if(pid<0)
    return -1;
  else if(pid!=0)
    exit(0);			/* terminate parent */
  else{
    /* We are the child */

    closeFile(stdIn);		/* close the three standard files */
    closeFile(stdOut);
    closeFile(stdErr);

    setsid();
    chdir("/");
    umask(0);			/* reset file creation mask */
  }
#endif
  return 0;
}
