#ifndef _OFFICEPOST_P_H_
#define _OFFICEPOST_P_H_

#ifdef WIN32
#include <winsock.h>
#else
#include <netinet/in.h>
#endif
typedef struct _message_ *MsgPo;

typedef struct _message_ {
  icmMsg msg;			/* The message itself */
  icmHandle recip;		/* Who is the message for */
  icmHandle snder;		/* Who was the sender */
  icmOption opts;		/* What options are in force for this msg */
  icmMsg pos;			/* Which segment of the message are we sending */
  long offset;			/* Where in that segment are we in sending it? */
  icmTruth sending;		/* Are we sending this message at the moment? */
  time_t lease;			/* lease hold on the message */
  MsgPo next;			/* Next message in the queue */
} MsgRec;

typedef struct _post_box_ {
  char host[MAXNAME];		/* Host name of this slot */
  icmIpPort addr;		/* what address are we using? */
  icmConn conn;			/* What connection is it based on? */
  long timeout;			/* Is there an auto-timeout on this connectio */
  time_t out_time;		/* When the socket should be timed-out */
  icmTruth timed_out;		/* Request for tiemout has occurred */
  MsgPo queue;			/* outgoing message queue */
  pthread_mutex_t mutex;	/* mutex to control access to the box */
  pthread_cond_t enqueued;	/* true when queue is non-empty */
  pthread_t writer;		/* This thread writes outgoing messages */
  pthread_t reader;		/* This thread reads incoming messages */
  icmTruth hasRdr;		/* True if there is an active reader */
  icmTruth hasWtr;		/* True if there is an active writer */
  long refCount;		/* Number of references to this box */
  pthread_cond_t dead;		/* Signalled when the last reference has gone */
  icmTruth terminating;		/* True when this box is terminating */
} PostBoxRec;

typedef enum {
  readEvent,			/* Input available */
  writeEvent,			/* Output ready */
  ioErrorEvent,			/* I/O exception */
  postEvent,			/* Message posting event */
  serverEvent,			/* A server command event */
  quitEvent			/* Termination event */
} eventType;

typedef struct _core_event_ *eventPo;

typedef struct _core_event_ {	/* A template for all types of event */
  eventType type;		/* What sort of event */
} CoreEventRec;

typedef struct _read_event_ *readEventPo;

typedef struct _read_event_ {
  CoreEventRec core;
  icmConn conn;			/* What connection is this input event from? */
} ReadEventRec;

#endif
