/*
 * Set up and clean up signals and signal processing
 */
#include "config.h"		/* Invoke configuration header */
#include <signal.h>
#include "daemon.h"

#ifdef HAVE_PTHREAD_SIGMASK
static pthread_t signal_thread;

static void *signalHdlr(void *arg);
#endif

static void sig_fatal(int sig)
{
  icmLogMsg("%d: fatal error",pthread_self());
  exit_daemon(1);
}

#if !defined(_WIN32) && !defined(__MWERKS__)
static void sig_bus(int sig)
{
  icmLogMsg("%d: bus error ",pthread_self());
  exit_daemon(1);
}
#endif

static void sig_seg(int sig)
{
  icmLogMsg("%d: segmentation fault",pthread_self());
  exit_daemon(1);
}

static void sig_int(int sig)
{
  icmLogMsg("%d: control-C interrupt",pthread_self());
  exit_daemon(1);
}

void setupsignals(void)
{
  signal(SIGFPE, sig_fatal);
  signal(SIGSEGV, sig_seg);
  signal(SIGINT, sig_int);

#ifdef ALLTRACE_
  return;
#endif

#ifdef HAVE_PTHREAD_SIGMASK
  pthread_create(&signal_thread,NULL,signalHdlr,NULL);
#endif

#if !defined(_WIN32) && !defined(__MWERKS__)
  signal(SIGBUS, sig_bus);
#endif
}

void blockSignals(void)
{
#ifdef HAVE_PTHREAD_SIGMASK
  sigset_t signals;

  sigemptyset(&signals);

  sigaddset(&signals,SIGINT);
  sigaddset(&signals,SIGTERM);

  pthread_sigmask(SIG_BLOCK,&signals,NULL); /* block SIGINT and SIGTERM */
#endif
}

#ifdef HAVE_PTHREAD_SIGMASK
static void *signalHdlr(void *arg)
{
  sigset_t signals;

  sigemptyset(&signals);

  sigaddset(&signals,SIGINT);
  sigaddset(&signals,SIGTERM);

  pthread_sigmask(SIG_BLOCK,&signals,NULL);
  
  while(icmYes){
    int signal;

    sigwait(&signals,&signal);

    switch(signal){
    case SIGINT:
      logMsg(icmLogFile,BOLD_ESC_ON"Control-C interrupt"BOLD_ESC_OFF);
      exit_daemon(1);
      break;
    case SIGTERM:
      logMsg(icmLogFile,BOLD_ESC_ON"Termination received"BOLD_ESC_OFF);
      break;
    default:
      logMsg(icmLogFile,BOLD_ESC_ON"Other signal %d received"BOLD_ESC_OFF,signal);
      exit_daemon(1);
    }
  }
}
#endif

void exit_daemon(int code)
{
  closePostOffice();
#ifdef SERVERTRACE
  if(traceSvr)
    icmLogMsg("exiting with code %d",code);
#endif
  exit(code);
}

